@@ -18,7 +18,7 @@ from typing import Iterable, Iterator, Union, Any, Optional, List, Dict, Corouti
 from furl import furl
 from pdb_profiling.log import Abclog
 import re
-from pdb_profiling.utils import init_semaphore
+from pdb_profiling.utils import init_semaphore, unsync_wrap
 
 
 class UnsyncFetch(Abclog):
@@ -57,9 +57,9 @@ class UnsyncFetch(Abclog):
     use_existing: bool = False
 
     @classmethod
-    @retry(wait=wait_random(max=10), stop=stop_after_attempt(3))
+    @retry(wait=wait_random(max=20), stop=stop_after_attempt(5))
     async def http_download(cls, method: str, info: Dict, path: str):
-        if cls.use_existing is True and os.path.exists(path):
+        if cls.use_existing is True and os.path.exists(path) and (os.stat(path).st_size > 0):
             return path
         cls.logger.debug(f"Start to download file: {info['url']}")
         async with aiohttp.ClientSession() as session:
@@ -82,12 +82,12 @@ class UnsyncFetch(Abclog):
                     raise Exception(mes)
 
     @classmethod
-    @retry(wait=wait_random(max=10), stop=stop_after_attempt(3))
+    @retry(wait=wait_random(max=20), stop=stop_after_attempt(5))
     async def ftp_download(cls, method: str, info: Dict, path: str):
         url = furl(info['url'])
         fileName = url.path.segments[-1]
         filePath = os.path.join(path, fileName)
-        if cls.use_existing is True and os.path.exists(filePath):
+        if cls.use_existing is True and os.path.exists(filePath) and (os.stat(filePath).st_size > 0):
             return filePath
         cls.logger.debug(f"Start to download file: {url}")  # info
         async with aioftp.ClientSession(url.host) as session:
@@ -131,16 +131,11 @@ class UnsyncFetch(Abclog):
     async def unsync_tasks(cls, tasks):
         return [await fob for fob in tqdm(asyncio.as_completed(tasks), total=len(tasks))]
 
-    @staticmethod
-    @unsync
-    def unsync_wrap(var):
-        return var
-
     @classmethod
     def single_task(cls, task, semaphore, to_do_func: Optional[Callable] = None, rate: float = 1.5) -> Unfuture:
         method, info, path = task
-        if cls.use_existing is True and os.path.exists(path) and os.path.isfile(path):
-            task = cls.unsync_wrap(path)
+        if cls.use_existing is True and os.path.exists(path) and os.path.isfile(path) and (os.stat(path).st_size > 0):
+            task = unsync_wrap(path)
         else:
             task = cls.fetch_file(semaphore, method, info, path, rate)
         if to_do_func is not None:
