@@ -14,10 +14,11 @@ from pdb_profiling.processors.pdbe.record import (
     PDBs,
     SIFTSs
     )
-from pdb_profiling.processors.pdbe.api import PDBeModelServer, PDBArchive,PDBVersioned
-from pdb_profiling.processors.uniprot.api import UniProtFASTA
+from pdb_profiling.processors.pdbe.api import PDBeModelServer, PDBArchive, PDBVersioned
+from pdb_profiling.processors.uniprot.api import UniProtFASTA, UniProtAPI
+from pdb_profiling.processors.uniprot.record import UniProts
 from pdb_profiling.processors.proteins.api import ProteinsAPI
-from pdb_profiling.processors.proteins.record import Identifier
+from pdb_profiling.processors.proteins.record import Identifier, Identifiers
 from pdb_profiling.processors.ensembl.api import EnsemblAPI
 from pdb_profiling.processors.eutils.api import EutilsAPI
 from pdb_profiling.processors.swissmodel.api import SMR
