@@ -9,7 +9,7 @@ import gzip
 import shutil
 from typing import Optional, Union, Dict, Tuple, Iterable, Iterator, List, Coroutine, NamedTuple, Callable, Generator
 from logging import Logger
-from pandas import read_csv, DataFrame, isna, Series
+from pandas import read_csv, DataFrame, isna, Series, concat
 import numpy as np
 from pathlib import Path
 from aiofiles import open as aiofiles_open
@@ -177,9 +177,14 @@ async def a_read_csv(path, read_mode='r',**kwargs):
     '''
     if isinstance(path, (Coroutine, Unfuture)):
         path = await path
-    async with aiofiles_open(path, read_mode) as file_io:
-        with StringIO(await file_io.read()) as text_io:
-            return read_csv(text_io, **kwargs)
+    if isinstance(path, (Path, str)):
+        async with aiofiles_open(path, read_mode) as file_io:
+            with StringIO(await file_io.read()) as text_io:
+                return read_csv(text_io, **kwargs)
+    elif isinstance(path, DataFrame):
+        return path
+    else:
+        return DataFrame(path, columns=kwargs.get('columns', None))
 
 
 async def a_load_json(path):
@@ -325,6 +330,7 @@ def iter_first(df: DataFrame, criteria: Callable[[NamedTuple], bool], **kwargs)
         if criteria(row):
             return row
 
+
 class MMCIF2DictPlus(dict):
     """
     Parse a mmCIF file and return a dictionary
@@ -542,7 +548,9 @@ async def a_seq_reader(path: Union[Unfuture, Union[Path, str]]):
         path = await path
     async with aiofiles_open(path, 'rt') as handle:
         header, content = fasta_pat.match(await handle.read()).groups()
-        return header, content.replace('\n', '')
+        content = content.replace('\n', '')
+        assert content != '', str(path)
+        return header, content
 
 
 @unsync
@@ -562,8 +570,8 @@ async def get_seqs_from_parser(res, identifiers:Optional[Iterable[str]]=None):
     return ret
 
 
-async def a_seq_parser(path: Union[Unfuture, Union[Path, str]]):
-    if isinstance(path, Unfuture):
+async def a_seq_parser(path: Union[Unfuture, Coroutine, Path, str]):
+    if isinstance(path, (Unfuture, Coroutine)):
         path = await path
     async with aiofiles_open(path, 'rt') as handle:
         header, content = None, ''
@@ -574,6 +582,7 @@ async def a_seq_parser(path: Union[Unfuture, Union[Path, str]]):
                 header, content = line, ''
             else:
                 content += line
+        assert header is not None, f"\npath: {path}\ncur_content: {content}"
         yield header.strip(), content.replace('\n', '')
 
 nu2aa_dict = {
@@ -609,9 +618,14 @@ def translate2aa(seq:str, check:bool=False):
 
 def unsync_run(arg):
     @unsync
-    async def unsync_wrap(arg):
+    async def s_unsync_wrap(arg):
         return await arg 
-    return unsync_wrap(arg).result()
+    return s_unsync_wrap(arg).result()
+
+
+@unsync
+def unsync_wrap(var):
+    return var
 
 
 class SeqRangeReader(object):
@@ -672,7 +686,7 @@ SEQ_DICT = {
 
 standardAA = list(SEQ_DICT.keys())
 
-standardNu = ['DA', 'DT', 'DC', 'DG', 'A', 'U', 'C', 'G']
+standardNu = ['DA', 'DT', 'DC', 'DG', 'DI', 'A', 'U', 'C', 'G', 'I']
 
 
 def range_len(lyst: Union[List, str, float]) -> int:
@@ -711,16 +725,30 @@ def lyst2range(lyst, add_end=1):
 def subtract_range(pdb_range: Union[str, Iterable], mis_range: Union[str, Iterable]) -> List:
     if isinstance(mis_range, float) or mis_range is None:
         return pdb_range
-    if len(pdb_range) == 0:
+    elif len(pdb_range) == 0:
         return ()
+    elif len(mis_range) == 0:
+        return pdb_range
     pdb_range_set = interval2set(pdb_range)
     mis_range_set = interval2set(mis_range)
     return to_interval(pdb_range_set - mis_range_set)
 
 
+def check_range(i) -> bool:
+    if isinstance(i, float) or (i is None) or (len(i) == 0) or (i == 'nan'):
+        return False
+    return True
+
+
 def add_range(left: Union[str, Iterable], right: Union[str, Iterable]) -> List:
-    if isinstance(right, float) or right is None or left is None or isinstance(left, float) or not right or not left or left == 'nan' or right == 'nan':
-        return None
+    check_left = check_range(left)
+    check_right = check_range(right)
+    if check_left and not check_right:
+        return left
+    elif not check_left and check_right:
+        return right
+    elif not check_left and not check_right:
+        return np.nan
     try:
         left_range_set = interval2set(left)
         right_range_set = interval2set(right)
@@ -733,7 +761,7 @@ def add_range(left: Union[str, Iterable], right: Union[str, Iterable]) -> List:
 
 def overlap_range(obs_range:Union[str, Iterable], unk_range: Union[str, Iterable]) -> List:
     if isinstance(unk_range, float) or unk_range is None:
-        return None
+        return ()
     '''
     obs_range_set = interval2set(obs_range)
     unk_range_set = interval2set(unk_range)
@@ -767,13 +795,13 @@ def get_seq_seg(seq, ranges):
         yield start, seq[start-1:end]
 
 
-def get_diff_index(lseq, lrange, rseq, rrange, on_left:bool=True):
+def get_diff_index(lseq, lrange, rseq, rrange):
     if isinstance(lrange, str):
         lrange = json.loads(lrange)
     if isinstance(rrange, str):
         rrange = json.loads(rrange)
     for (lstart, lseg), (rstart, rseg) in zip(get_seq_seg(lseq, lrange), get_seq_seg(rseq, rrange)):
-        yield from (lstart+index if on_left else rstart+index for index, (r1, r2) in enumerate(zip(lseg, rseg)) if r1 != r2)
+        yield from ((lstart+index, rstart+index) for index, (r1, r2) in enumerate(zip(lseg, rseg)) if r1 != r2)
 
 
 def red_seq_seg(seq, ranges):
@@ -784,6 +812,14 @@ def red_seq_seg(seq, ranges):
     yield seq[end:]
 
 
+def outside_range(pdb_range: Union[str, Iterable], seqres_len: int):
+    pdb_range = json.loads(pdb_range) if isinstance(pdb_range, str) else pdb_range
+    out_head = pdb_range[0][0] - 1
+    out_tail = pdb_range[-1][-1] + 1
+    ret = [[1, out_head], [out_tail, seqres_len]]
+    return [i for i in ret if i[0]<=i[1]]
+
+
 def outside_range_len(pdb_range: Union[str, Iterable], seqres_len: int, omit: int = 5) -> int:
     if isinstance(pdb_range, str):
         lyst = json.loads(pdb_range)
@@ -816,21 +852,22 @@ def get_range_diff(lyst_a: Union[str, List, Tuple], lyst_b: Union[str, List, Tup
     return array_a - array_b
 
 
-def select_range(ranges, indexes, cutoff=0.2, skip_index=[]):
+def select_range(ranges, indexes, cutoff=0.2, skip_index=[], selected_ranges=None):
     select_index = []
+    selected_ranges = [] if selected_ranges is None else selected_ranges
     def unit(cur_index):
         if cur_index in skip_index:
             return
         cur_range = ranges[cur_index]
         cur_range = json.loads(cur_range) if isinstance(cur_range, str) else cur_range
-        for selected in select_index:
-            selected_range = ranges[selected]
+        for selected_range in selected_ranges:
             selected_range = json.loads(selected_range) if isinstance(selected_range, str) else selected_range
             score = overlap.similarity(lyst2range(cur_range),
                                lyst2range(selected_range))
             if score > cutoff:
                 return
         select_index.append(cur_index)
+        selected_ranges.append(cur_range)
 
     for index in indexes:
         unit(index)
@@ -898,3 +935,14 @@ def select_he_range(Entry_1, Entry_2, ranges1, ranges2, indexes, cutoff=0.2, ski
     for index in indexes:
         unit(index)
     return select_index
+
+
+def dumpsParams(params: Dict) -> str:
+    return '&'.join(f'{key}={value}' for key, value in params.items())
+
+
+@unsync
+async def a_concat(pathes, sep='\t', sort=False, ignore_index=True, columns=None):
+    if isinstance(pathes, (Unfuture, Coroutine)):
+        pathes = await pathes
+    return concat([await a_read_csv((await path) if isinstance(Unfuture, Coroutine) else path, sep=sep, columns=columns) for path in pathes], sort=sort, ignore_index=ignore_index)
